Struct double_dot_state::AmbientLight
pub struct AmbientLight {
pub color: Color,
pub brightness: f32,
}Expand description
An ambient light, which lights the entire scene equally.
Fields
color: Colorbrightness: f32A direct scale factor multiplied with color before being passed to the shader.