Struct double_dot_state::CardinalSpline
pub struct CardinalSpline<P>where
P: Point,{ /* private fields */ }Expand description
A spline interpolated continuously across the nearest four control points, with the position of the curve specified at every control point and the tangents computed automatically.
Note the Catmull-Rom spline is a special case of Cardinal spline where the tension is 0.5.
Interpolation
The curve passes through every control point.
Tangency
Automatically defined at each control point.
Continuity
C1 continuous.
Usage
let points = [
vec2(-1.0, -20.0),
vec2(3.0, 2.0),
vec2(5.0, 3.0),
vec2(9.0, 8.0),
];
let cardinal = CardinalSpline::new(0.3, points).to_curve();
let positions: Vec<_> = cardinal.iter_positions(100).collect();Implementations
impl<P> CardinalSpline<P>where
P: Point,
impl<P> CardinalSpline<P>where
P: Point,
pub fn new(tension: f32, control_points: impl Into<Vec<P>>) -> CardinalSpline<P>
pub fn new(tension: f32, control_points: impl Into<Vec<P>>) -> CardinalSpline<P>
Build a new Cardinal spline.
pub fn new_catmull_rom(control_points: impl Into<Vec<P>>) -> CardinalSpline<P>
pub fn new_catmull_rom(control_points: impl Into<Vec<P>>) -> CardinalSpline<P>
Build a new Catmull-Rom spline, the special case of a Cardinal spline where tension = 1/2.