pub struct Gamepad { /* private fields */ }Expand description
Stores a connected gamepad’s metadata such as the name and its GamepadButton and GamepadAxis.
An entity with this component is spawned automatically after GamepadConnectionEvent
and updated by gamepad_event_processing_system.
See also GamepadSettings for configuration.
Examples
fn gamepad_usage_system(gamepads: Query<(&Name, &Gamepad)>) {
for (name, gamepad) in &gamepads {
println!("{name}");
if gamepad.just_pressed(GamepadButton::North) {
println!("{} just pressed North", name)
}
if let Some(left_stick_x) = gamepad.get(GamepadAxis::LeftStickX) {
println!("left stick X: {}", left_stick_x)
}
}
}Implementations
Sourceimpl Gamepad
impl Gamepad
Sourcepub fn vendor_id(&self) -> Option<u16>
pub fn vendor_id(&self) -> Option<u16>
Returns the USB vendor ID as assigned by the USB-IF, if available.
Sourcepub fn product_id(&self) -> Option<u16>
pub fn product_id(&self) -> Option<u16>
Returns the USB product ID as assigned by the vendor, if available.
Sourcepub fn get(&self, input: impl Into<GamepadInput>) -> Option<f32>
pub fn get(&self, input: impl Into<GamepadInput>) -> Option<f32>
Returns the analog data of the provided GamepadAxis or GamepadButton.
Sourcepub fn get_unclamped(&self, input: impl Into<GamepadInput>) -> Option<f32>
pub fn get_unclamped(&self, input: impl Into<GamepadInput>) -> Option<f32>
Returns the unclamped analog data of the provided GamepadAxis or GamepadButton.
This value may be outside the Axis::MIN and Axis::MAX range.
Sourcepub fn left_stick(&self) -> Vec2
pub fn left_stick(&self) -> Vec2
Returns the left stick as a Vec2.
Sourcepub fn right_stick(&self) -> Vec2
pub fn right_stick(&self) -> Vec2
Returns the right stick as a Vec2.
Sourcepub fn pressed(&self, button_type: GamepadButton) -> bool
pub fn pressed(&self, button_type: GamepadButton) -> bool
Returns true if the GamepadButton has been pressed.
Sourcepub fn any_pressed(
&self,
button_inputs: impl IntoIterator<Item = GamepadButton>,
) -> bool
pub fn any_pressed( &self, button_inputs: impl IntoIterator<Item = GamepadButton>, ) -> bool
Returns true if any item in the GamepadButton iterator has been pressed.
Sourcepub fn all_pressed(
&self,
button_inputs: impl IntoIterator<Item = GamepadButton>,
) -> bool
pub fn all_pressed( &self, button_inputs: impl IntoIterator<Item = GamepadButton>, ) -> bool
Returns true if all items in the GamepadButton iterator have been pressed.
Sourcepub fn just_pressed(&self, button_type: GamepadButton) -> bool
pub fn just_pressed(&self, button_type: GamepadButton) -> bool
Returns true if the GamepadButton has been pressed during the current frame.
Note: This function does not imply information regarding the current state of ButtonInput::pressed or ButtonInput::just_released.
Sourcepub fn any_just_pressed(
&self,
button_inputs: impl IntoIterator<Item = GamepadButton>,
) -> bool
pub fn any_just_pressed( &self, button_inputs: impl IntoIterator<Item = GamepadButton>, ) -> bool
Returns true if any item in the GamepadButton iterator has been pressed during the current frame.
Sourcepub fn all_just_pressed(
&self,
button_inputs: impl IntoIterator<Item = GamepadButton>,
) -> bool
pub fn all_just_pressed( &self, button_inputs: impl IntoIterator<Item = GamepadButton>, ) -> bool
Returns true if all items in the GamepadButton iterator have been just pressed.
Sourcepub fn just_released(&self, button_type: GamepadButton) -> bool
pub fn just_released(&self, button_type: GamepadButton) -> bool
Returns true if the GamepadButton has been released during the current frame.
Note: This function does not imply information regarding the current state of ButtonInput::pressed or ButtonInput::just_pressed.
Sourcepub fn any_just_released(
&self,
button_inputs: impl IntoIterator<Item = GamepadButton>,
) -> bool
pub fn any_just_released( &self, button_inputs: impl IntoIterator<Item = GamepadButton>, ) -> bool
Returns true if any item in the GamepadButton iterator has just been released.
Sourcepub fn all_just_released(
&self,
button_inputs: impl IntoIterator<Item = GamepadButton>,
) -> bool
pub fn all_just_released( &self, button_inputs: impl IntoIterator<Item = GamepadButton>, ) -> bool
Returns true if all items in the GamepadButton iterator have just been released.
Sourcepub fn get_pressed(&self) -> impl Iterator<Item = &GamepadButton>
pub fn get_pressed(&self) -> impl Iterator<Item = &GamepadButton>
Returns an iterator over all digital buttons that are pressed.
Sourcepub fn get_just_pressed(&self) -> impl Iterator<Item = &GamepadButton>
pub fn get_just_pressed(&self) -> impl Iterator<Item = &GamepadButton>
Returns an iterator over all digital buttons that were just pressed.
Sourcepub fn get_just_released(&self) -> impl Iterator<Item = &GamepadButton>
pub fn get_just_released(&self) -> impl Iterator<Item = &GamepadButton>
Returns an iterator over all digital buttons that were just released.
Sourcepub fn get_analog_axes(&self) -> impl Iterator<Item = &GamepadInput>
pub fn get_analog_axes(&self) -> impl Iterator<Item = &GamepadInput>
Returns an iterator over all analog axes.
Sourcepub fn digital(&self) -> &ButtonInput<GamepadButton>
pub fn digital(&self) -> &ButtonInput<GamepadButton>
ButtonInput of GamepadButton representing their digital state.
Sourcepub fn digital_mut(&mut self) -> &mut ButtonInput<GamepadButton>
pub fn digital_mut(&mut self) -> &mut ButtonInput<GamepadButton>
Mutable ButtonInput of GamepadButton representing their digital state. Useful for mocking inputs.
Sourcepub fn analog(&self) -> &Axis<GamepadInput>
pub fn analog(&self) -> &Axis<GamepadInput>
Axis of GamepadButton representing their analog state.
Sourcepub fn analog_mut(&mut self) -> &mut Axis<GamepadInput>
pub fn analog_mut(&mut self) -> &mut Axis<GamepadInput>
Mutable Axis of GamepadButton representing their analog state. Useful for mocking inputs.