Struct double_dot_state::OnUpdate
pub struct OnUpdate<S>(pub S)
where
S: States;Expand description
Tuple Fields
0: STrait Implementations
impl<S> PartialEq for OnUpdate<S>
impl<S> PartialEq for OnUpdate<S>
impl<S> SystemSet for OnUpdate<S>
impl<S> SystemSet for OnUpdate<S>
fn is_base(&self) -> bool
fn is_base(&self) -> bool
Returns
true if this set is a “base system set”. Systems
can only belong to one base set at a time. Systems and Sets
can only be added to base sets using specialized in_base_set
APIs. This enables “mutually exclusive” behaviors. It also
enables schedules to have a “default base set”, which can be used
to apply default configuration to systems.fn dyn_clone(&self) -> Box<dyn SystemSet>
fn dyn_clone(&self) -> Box<dyn SystemSet>
Creates a boxed clone of the label corresponding to this system set.
fn system_type(&self) -> Option<TypeId>
fn system_type(&self) -> Option<TypeId>
Returns
Some if this system set is a [SystemTypeSet].impl<S> Eq for OnUpdate<S>
impl<S> FreeSystemSet for OnUpdate<S>
impl<S> StructuralEq for OnUpdate<S>where
S: States,
impl<S> StructuralPartialEq for OnUpdate<S>where
S: States,